import numpy as np
import matplotlib.pyplot as plt

class SubSer3Cat():
    def __init__(self, start_v=None, start_delta=None,
                 min_delta=None, cond=None, ser_id=None):
        self.v = start_v
        self.delta = start_delta
        self.min_delta = min_delta
        self.cond = cond
        self.ser_id = ser_id

    def next_state(self, response):
        if response <= 1:
            self.v += self.delta
            self.cond = 'up'
        elif response >= 3:
            self.v -= self.delta
            self.cond = 'down'
        else:
            if self.cond == 'up':
                self.v += self.delta
            else:
                self.v -= self.delta
        self.delta *= 0.5
        if self.delta < self.min_delta:
            self.delta = self.min_delta

ser = ['A']*2 + ['B']*2
 
fout_nm = input('出力ファイル名(.txt) = ')
fout = open(fout_nm, 'w')

while True:
    Start_A = float(input('初期値（大きい方）= '))
    Start_B = float(input('初期値（小さい方）= '))
    Start_delta = float(input('Δ値（初期値）= '))
    Min_delta = float(input('Δ値（下限値）= '))
    N_blocks = int(input('ブロック数（１ブロック=４試行）= '))
    print('初期値（大きい方）=', Start_A)
    print('初期値（小さい方）=', Start_B)
    print('Δ値（初期値）=', Start_delta)
    print('ブロック数 =', N_blocks)
    ck = input('OK? (y/n) = ')
    if ck in ['y', 'Y']:
        break

data = []

serA = SubSer3Cat(start_v = Start_A, start_delta = Start_delta, min_delta = Min_delta,
                  cond='down', ser_id='A')

serB = SubSer3Cat(start_v = Start_B, start_delta = Start_delta, min_delta = Min_delta,
                  cond='up', ser_id='B')

t = 0
for b in range(N_blocks):
    np.random.shuffle(ser)
    for i in range(4):
        t += 1
        print('\n試行：{0:}/{1:}'.format(t, N_blocks*4))
        if ser[i] == 'A':
            serX = serA
        else:
            serX = serB
            
        print('刺激値：', serX.v)
        res = int(input("\n反応(1:小・弱, 2:不明・等しい, 3:大・強)? "))
        data.append([f'{serX.ser_id}{t}', serX.v, res])
        serX.next_state(res)
            
print('{0:>15}{1:>15}{2:>15}'.format('trial', 'stimulus', 'response'))
fout.write('{0:>15}{1:>15}{2:>15}'.format('trial', 'stimulus', 'response'))

for i, v in enumerate(data):
    print('{0:>15}{1:>15}{2:>15}'.format(v[0], v[1], v[2]))
    fout.write('\n{0:>15}{1:>15}{2:>15}'.format(v[0], v[1], v[2]))
    plt.plot([i], [v[1]],'^' if v[2] == 3 else 'v' if v[2] == 1 else 'o',
             c = 'b' if v[0][0] == 'A' else 'g')

plt.plot([], 's', c = 'b', label = 'Ser.A')
plt.plot([], 's', c = 'g', label = 'Ser.B')
plt.plot([], '^', c = 'k', label = '3')
plt.plot([], 'o', c = 'k', label = '2')
plt.plot([], 'v', c = 'k', label = '1')
plt.title('Up-down Method')
plt.xlabel('Trial')
plt.ylabel('Comp. St.')
plt.legend()
plt.savefig('Fig' + fout_nm[:-4] + '.png')
plt.show()

print('\nFig' + fout_nm[:-4] + '.png was saved.')
fout.close()
print(fout_nm + '.was saved.\n')
